import Queue

from rdoclient import RandomOrgClient


def get_capacity_left(api_key):
    r = RandomOrgClient(api_key)
    bits = r.get_bits_left()
    reqs = r.get_requests_left()
    return reqs, bits


def has_capacity(api_key):
    reqsbits = get_capacity_left(api_key)
    if reqsbits[0] < 5 and reqsbits[1] < 300:
        return False
    return True


def test_connection(api_key):
    try:
        print(api_key)
        left = get_capacity_left(api_key)
        print(left)
        return True
    except TypeError as te:
        print(te)

    except ValueError as ve:
        print(ve)

    return False


def initialize_client(api_key, blocking_timeout=None, http_timeout=None, cached=False, live_results=False, **kwargs):
    r = None
    c = None
    if has_capacity(api_key):
        if blocking_timeout and http_timeout:
            r = RandomOrgClient(api_key, blocking_timeout, http_timeout)

        if cached:
            if r is None:
                r = RandomOrgClient(api_key, blocking_timeout=60.0 * 60.0, http_timeout=30.0)
            cache_size = kwargs['cache_size']
            cache_min = kwargs['cache_min']
            cache_max = kwargs['cache_max']
            # print(kwargs)
            c = r.create_integer_cache(cache_size, cache_min, cache_max)


        elif live_results:
            r = RandomOrgClient(api_key, blocking_timeout=0.0, http_timeout=10.0, serialized=False)
    return r, c


def request_random_integers(amount, min_int, max_int, client, cache=None):
    if cache is not None:
        # print(catche)
        try:
            return cache.get()
        except Queue.Empty:
            # print('queue is empty')
            pass
    else:
        return client.generate_integers(amount, min_int, max_int)


# key = verify_api_key()
# print(get_capacity_left(key))


if __name__ == '__main__':
    t = initialize_client(True, cache_size=5, cache_min=1, cache_max=10)



import re

from dice import Dice, BufferedDice

dice_pattern = re.compile('\d+(d|D)\d+')


def extract_dice_from_string(s):
    s = str.lower(s)
    arr = s.split('d')
    dices = arr[0]
    sides = arr[1]
    return dices, sides


def throw_the_dice(command):
    dices, sides = extract_dice_from_string(command)
    d = Dice(sides=sides, dices=dices)
    print(d.roll)
    return dices, sides


def simple_dice_app():
    command = 'Y'
    last_dices = 0
    last_sides = 0
    prompt = 'roll a dice by entering e.g. 1d6 or enter Q to quit\n'
    while command != 'Q':
        if last_dices != 0:
            prompt = "roll " + str(last_dices) + "d" + str(
                last_sides) + " again by pressing enter, type another dice or Q to Quit\n"
        command = raw_input(prompt)
        if str(command) == '':
            throw_the_dice(str(last_dices) + 'd' + str(last_sides))
        elif dice_pattern.match(command):
            dices, sides = throw_the_dice(command)
            last_dices = dices
            last_sides = sides
        elif str(command) != str('Q'):
            print('invalid input, try "3d6" without "-signs')


def buffered_dice_app():
    command = 'Y'

    dice = None
    while dice is None:
        prompt = 'What kind of dice do you want to throw? input a valid dice (e.g. 1d6) or Q for quit\n'
        kind = raw_input(prompt)
        if dice_pattern.match(kind):
            dices, sides = extract_dice_from_string(kind)
            dice = BufferedDice(dices, sides)
        elif str(kind) == 'Q':
            break
        else:
            print('invalid input type')

    while command != 'Q':
        prompt = 'press enter to roll another ' + str(dice)
        command = raw_input(prompt)
        print(dice.roll)


if __name__ == '__main__':
    buffered_dice_app()
    # simple_dice_app()



from peewee import SqliteDatabase, Model, CharField, BooleanField, DoesNotExist

db = SqliteDatabase('apikey.db')


class ApiKey(Model):
    key = CharField(unique=True, primary_key=True)
    verified = BooleanField(default=False)

    @staticmethod
    def get_key():
        try:
            q = ApiKey.get()
            return q
        except DoesNotExist as e:
            print('no api-key')
            return None

    @staticmethod
    def save_key(key):
        return ApiKey.create(key=key)


    @staticmethod
    def delete_key():
        q = ApiKey.delete()
        q.execute()

    @staticmethod
    def update_key_to_valid():
        g = ApiKey.get_key()
        g.verified = True
        g.save()

    class Meta:
        database = db


db.create_tables([ApiKey], safe=True)

if __name__ == '__main__':
    ApiKey.save_key('dlsls')
    ApiKey.delete_key()



import diceroller
from database import ApiKey
from ranclient import test_connection


def ask_api_key():
    # https://stackoverflow.com/questions/4960208/python-2-7-getting-user-input-and-manipulating-as-string-without-quotations
    api_key = str(raw_input('Enter valid api-key: '))
    final_key = ApiKey.save_key(api_key)
    return final_key

def verify_api_key():
    db_key = ApiKey.get_key()

    if db_key is None:
        db_key = ask_api_key()
    api_key = db_key.key
    verified = db_key.verified

    if verified != True:
        while test_connection(api_key) != True:
            api_key = ask_api_key()
        ApiKey.update_key_to_valid()

    print('api-key verification complete')
    return api_key


if __name__ == '__main__':
    verify_api_key()
    diceroller.simple_dice_app()



import time

import database
import ranclient


# https://stackoverflow.com/a/14917171/1754089
def nested_sum(L):
    return sum(nested_sum(x) if isinstance(x, list) else x for x in L)


class Dice(object):
    def __init__(self, dices, sides):
        super(Dice, self).__init__()
        self.__dices = int(dices)
        self.__sides = int(sides)
        self.__client = None
        api_key = database.ApiKey.get_key()
        self.api_key = api_key.key

    @property
    def client(self):
        if self.__client:
            return self.__client
        else:
            client, cache = ranclient.initialize_client(self.api_key, live_results=True)
            self.__client = client
            return client

    @client.setter
    def client(self, new_value):
        if self.__client:
            pass
        else:
            self.__client = new_value

    @property
    def sides(self):
        return self.__sides

    @property
    def dices(self):
        return self.__dices

    @property
    def roll(self):

        arr = ranclient.request_random_integers(self.__dices, 1, self.__sides, self.client)
        result = nested_sum(arr)
        return result, arr

    def roll_times(self, n):
        results = []
        for i in range(0, n):
            results.append(self.roll)
        return results

    def __repr__(self):
        return str(self.dices) + "d" + str(self.sides)


class BufferedDice(Dice):
    def __init__(self, dices, sides, cache_size=10):
        super(BufferedDice, self).__init__(dices, sides)
        client, cache = ranclient.initialize_client(self.api_key, cached=True, cache_min=1, cache_max=sides,
                                                    cache_size=cache_size)
        self.client = client
        self.__cache = cache
        self.fifo_of_integers = []

    @property
    def roll(self):
        """Rolls the dice

        :return: tuple containing (sum of thrown dices and array with individual dices
        """
        if len(self.fifo_of_integers) < self.dices:
            # print('getting more dices')
            success = False
            while not success:
                try:
                    arr = ranclient.request_random_integers(self.dices, 1, self.sides, self.client, self.__cache)
                    for i in arr:
                        self.fifo_of_integers.append(i)
                        success = True
                except TypeError:
                    time.sleep(0.1)

        ds = []
        for i in range(0, self.dices):
            ds.append(self.fifo_of_integers.pop())

        result = sum(ds)
        return result, ds



from distutils.core import setup

setup(
    name='truerandomdice',
    version='0.1',
    packages=['truerandomdice'],
    url='https://github.com/EarthModule/TrulyRandomDiceThrower',
    download_url='https://github.com/EarthModule/TrulyRandomDiceThrower/releases/tag/0.1',
    license='',
    author='toni nurmi',
    author_email='toni.nurmi@hotmail.com',
    description=''
)
